=begin
#Velo Payments APIs

### Terms and Definitions  Throughout this document and the Velo platform the following terms are used:  * **Payor.** An entity (typically a corporation) which wishes to pay funds to one or more payees via a payout. * **Payee.** The recipient of funds paid out by a payor. * **Payment.** A single transfer of funds from a payor to a payee. * **Payout.** A batch of Payments, typically used by a payor to logically group payments (e.g. by business day). Technically there need be no relationship between the payments in a payout - a single payout can contain payments to multiple payees and/or multiple payments to a single payee. * **Sandbox.** An integration environment provided by Velo Payments which offers a similar API experience to the production environment, but all funding and payment events are simulated, along with many other services such as OFAC sanctions list checking.  ## Overview  The Velo Payments API allows a payor to perform a number of operations. The following is a list of the main capabilities in a natural order of execution:  * Authenticate with the Velo platform * Maintain a collection of payees * Query the payor’s current balance of funds within the platform and perform additional funding * Issue payments to payees * Query the platform for a history of those payments  This document describes the main concepts and APIs required to get up and running with the Velo Payments platform. It is not an exhaustive API reference. For that, please see the separate Velo Payments API Reference.  ## API Considerations  The Velo Payments API is REST based and uses the JSON format for requests and responses.  Most calls are secured using OAuth 2 security and require a valid authentication access token for successful operation. See the Authentication section for details.  Where a dynamic value is required in the examples below, the {token} format is used, suggesting that the caller needs to supply the appropriate value of the token in question (without including the { or } characters).  Where curl examples are given, the –d @filename.json approach is used, indicating that the request body should be placed into a file named filename.json in the current directory. Each of the curl examples in this document should be considered a single line on the command-line, regardless of how they appear in print.  ## Authenticating with the Velo Platform  Once Velo backoffice staff have added your organization as a payor within the Velo platform sandbox, they will create you a payor Id, an API key and an API secret and share these with you in a secure manner.  You will need to use these values to authenticate with the Velo platform in order to gain access to the APIs. The steps to take are explained in the following:  create a string comprising the API key (e.g. 44a9537d-d55d-4b47-8082-14061c2bcdd8) and API secret (e.g. c396b26b-137a-44fd-87f5-34631f8fd529) with a colon between them. E.g. 44a9537d-d55d-4b47-8082-14061c2bcdd8:c396b26b-137a-44fd-87f5-34631f8fd529  base64 encode this string. E.g.: NDRhOTUzN2QtZDU1ZC00YjQ3LTgwODItMTQwNjFjMmJjZGQ4OmMzOTZiMjZiLTEzN2EtNDRmZC04N2Y1LTM0NjMxZjhmZDUyOQ==  create an HTTP **Authorization** header with the value set to e.g. Basic NDRhOTUzN2QtZDU1ZC00YjQ3LTgwODItMTQwNjFjMmJjZGQ4OmMzOTZiMjZiLTEzN2EtNDRmZC04N2Y1LTM0NjMxZjhmZDUyOQ==  perform the Velo authentication REST call using the HTTP header created above e.g. via curl:  ```   curl -X POST \\   -H \"Content-Type: application/json\" \\   -H \"Authorization: Basic NDRhOTUzN2QtZDU1ZC00YjQ3LTgwODItMTQwNjFjMmJjZGQ4OmMzOTZiMjZiLTEzN2EtNDRmZC04N2Y1LTM0NjMxZjhmZDUyOQ==\" \\   'https://api.sandbox.velopayments.com/v1/authenticate?grant_type=client_credentials' ```  If successful, this call will result in a **200** HTTP status code and a response body such as:  ```   {     \"access_token\":\"19f6bafd-93fd-4747-b229-00507bbc991f\",     \"token_type\":\"bearer\",     \"expires_in\":1799,     \"scope\":\"...\"   } ``` ## API access following authentication Following successful authentication, the value of the access_token field in the response (indicated in green above) should then be presented with all subsequent API calls to allow the Velo platform to validate that the caller is authenticated.  This is achieved by setting the HTTP Authorization header with the value set to e.g. Bearer 19f6bafd-93fd-4747-b229-00507bbc991f such as the curl example below:  ```   -H \"Authorization: Bearer 19f6bafd-93fd-4747-b229-00507bbc991f \" ```  If you make other Velo API calls which require authorization but the Authorization header is missing or invalid then you will get a **401** HTTP status response. 

The version of the OpenAPI document: 2.20.118

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.3.1-SNAPSHOT

=end

require 'cgi'

module VeloPayments
  class FundingManagerApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # Create Funding Request
    # Instruct a funding request to transfer funds from the payor’s funding bank to the payor’s balance held within Velo.
    # @param source_account_id [String] Source account id
    # @param funding_request_v1 [FundingRequestV1] Body to included ammount to be funded
    # @param [Hash] opts the optional parameters
    # @return [nil]
    def create_ach_funding_request(source_account_id, funding_request_v1, opts = {})
      create_ach_funding_request_with_http_info(source_account_id, funding_request_v1, opts)
      nil
    end

    # Create Funding Request
    # Instruct a funding request to transfer funds from the payor’s funding bank to the payor’s balance held within Velo.
    # @param source_account_id [String] Source account id
    # @param funding_request_v1 [FundingRequestV1] Body to included ammount to be funded
    # @param [Hash] opts the optional parameters
    # @return [Array<(nil, Integer, Hash)>] nil, response status code and response headers
    def create_ach_funding_request_with_http_info(source_account_id, funding_request_v1, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.create_ach_funding_request ...'
      end
      # verify the required parameter 'source_account_id' is set
      if @api_client.config.client_side_validation && source_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'source_account_id' when calling FundingManagerApi.create_ach_funding_request"
      end
      # verify the required parameter 'funding_request_v1' is set
      if @api_client.config.client_side_validation && funding_request_v1.nil?
        fail ArgumentError, "Missing the required parameter 'funding_request_v1' when calling FundingManagerApi.create_ach_funding_request"
      end
      # resource path
      local_var_path = '/v1/sourceAccounts/{sourceAccountId}/achFundingRequest'.sub('{' + 'sourceAccountId' + '}', CGI.escape(source_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(funding_request_v1) 

      # return_type
      return_type = opts[:return_type] 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#create_ach_funding_request\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Create Funding Request
    # Instruct a funding request to transfer funds from the payor’s funding bank to the payor’s balance held within Velo  (202 - accepted, 400 - invalid request body, 404 - source account not found).
    # @param source_account_id [String] Source account id
    # @param funding_request_v2 [FundingRequestV2] Body to included ammount to be funded
    # @param [Hash] opts the optional parameters
    # @return [nil]
    def create_funding_request(source_account_id, funding_request_v2, opts = {})
      create_funding_request_with_http_info(source_account_id, funding_request_v2, opts)
      nil
    end

    # Create Funding Request
    # Instruct a funding request to transfer funds from the payor’s funding bank to the payor’s balance held within Velo  (202 - accepted, 400 - invalid request body, 404 - source account not found).
    # @param source_account_id [String] Source account id
    # @param funding_request_v2 [FundingRequestV2] Body to included ammount to be funded
    # @param [Hash] opts the optional parameters
    # @return [Array<(nil, Integer, Hash)>] nil, response status code and response headers
    def create_funding_request_with_http_info(source_account_id, funding_request_v2, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.create_funding_request ...'
      end
      # verify the required parameter 'source_account_id' is set
      if @api_client.config.client_side_validation && source_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'source_account_id' when calling FundingManagerApi.create_funding_request"
      end
      # verify the required parameter 'funding_request_v2' is set
      if @api_client.config.client_side_validation && funding_request_v2.nil?
        fail ArgumentError, "Missing the required parameter 'funding_request_v2' when calling FundingManagerApi.create_funding_request"
      end
      # resource path
      local_var_path = '/v2/sourceAccounts/{sourceAccountId}/fundingRequest'.sub('{' + 'sourceAccountId' + '}', CGI.escape(source_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(funding_request_v2) 

      # return_type
      return_type = opts[:return_type] 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#create_funding_request\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Funding Account
    # Get Funding Account by ID
    # @param funding_account_id [String] 
    # @param [Hash] opts the optional parameters
    # @option opts [Boolean] :sensitive  (default to false)
    # @return [FundingAccountResponse]
    def get_funding_account(funding_account_id, opts = {})
      data, _status_code, _headers = get_funding_account_with_http_info(funding_account_id, opts)
      data
    end

    # Get Funding Account
    # Get Funding Account by ID
    # @param funding_account_id [String] 
    # @param [Hash] opts the optional parameters
    # @option opts [Boolean] :sensitive 
    # @return [Array<(FundingAccountResponse, Integer, Hash)>] FundingAccountResponse data, response status code and response headers
    def get_funding_account_with_http_info(funding_account_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_funding_account ...'
      end
      # verify the required parameter 'funding_account_id' is set
      if @api_client.config.client_side_validation && funding_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'funding_account_id' when calling FundingManagerApi.get_funding_account"
      end
      # resource path
      local_var_path = '/v1/fundingAccounts/{fundingAccountId}'.sub('{' + 'fundingAccountId' + '}', CGI.escape(funding_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'sensitive'] = opts[:'sensitive'] if !opts[:'sensitive'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'FundingAccountResponse' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_funding_account\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Funding Accounts
    # Get the source accounts.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :payor_id 
    # @option opts [String] :source_account_id 
    # @option opts [Integer] :page Page number. Default is 1. (default to 1)
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100. (default to 25)
    # @option opts [String] :sort  (default to 'accountName:asc')
    # @option opts [Boolean] :sensitive  (default to false)
    # @return [ListFundingAccountsResponse]
    def get_funding_accounts(opts = {})
      data, _status_code, _headers = get_funding_accounts_with_http_info(opts)
      data
    end

    # Get Funding Accounts
    # Get the source accounts.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :payor_id 
    # @option opts [String] :source_account_id 
    # @option opts [Integer] :page Page number. Default is 1.
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100.
    # @option opts [String] :sort 
    # @option opts [Boolean] :sensitive 
    # @return [Array<(ListFundingAccountsResponse, Integer, Hash)>] ListFundingAccountsResponse data, response status code and response headers
    def get_funding_accounts_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_funding_accounts ...'
      end
      # resource path
      local_var_path = '/v1/fundingAccounts'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'payorId'] = opts[:'payor_id'] if !opts[:'payor_id'].nil?
      query_params[:'sourceAccountId'] = opts[:'source_account_id'] if !opts[:'source_account_id'].nil?
      query_params[:'page'] = opts[:'page'] if !opts[:'page'].nil?
      query_params[:'pageSize'] = opts[:'page_size'] if !opts[:'page_size'].nil?
      query_params[:'sort'] = opts[:'sort'] if !opts[:'sort'].nil?
      query_params[:'sensitive'] = opts[:'sensitive'] if !opts[:'sensitive'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'ListFundingAccountsResponse' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_funding_accounts\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Fundings for Payor
    # Get a list of Fundings for a payor. 
    # @param [Hash] opts the optional parameters
    # @option opts [String] :payor_id The account owner Payor ID
    # @option opts [Integer] :page Page number. Default is 1. (default to 1)
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100. (default to 25)
    # @option opts [String] :sort List of sort fields. Example: &#x60;&#x60;&#x60;?sort&#x3D;destinationCurrency:asc,destinationAmount:asc&#x60;&#x60;&#x60; Default is no sort. The supported sort fields are: dateTime and amount. 
    # @return [GetFundingsResponse]
    def get_fundings_v1(opts = {})
      data, _status_code, _headers = get_fundings_v1_with_http_info(opts)
      data
    end

    # Get Fundings for Payor
    # Get a list of Fundings for a payor. 
    # @param [Hash] opts the optional parameters
    # @option opts [String] :payor_id The account owner Payor ID
    # @option opts [Integer] :page Page number. Default is 1.
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100.
    # @option opts [String] :sort List of sort fields. Example: &#x60;&#x60;&#x60;?sort&#x3D;destinationCurrency:asc,destinationAmount:asc&#x60;&#x60;&#x60; Default is no sort. The supported sort fields are: dateTime and amount. 
    # @return [Array<(GetFundingsResponse, Integer, Hash)>] GetFundingsResponse data, response status code and response headers
    def get_fundings_v1_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_fundings_v1 ...'
      end
      # resource path
      local_var_path = '/v1/paymentaudit/fundings'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'payorId'] = opts[:'payor_id'] if !opts[:'payor_id'].nil?
      query_params[:'page'] = opts[:'page'] if !opts[:'page'].nil?
      query_params[:'pageSize'] = opts[:'page_size'] if !opts[:'page_size'].nil?
      query_params[:'sort'] = opts[:'sort'] if !opts[:'sort'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'GetFundingsResponse' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_fundings_v1\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get details about given source account.
    # Get details about given source account.
    # @param source_account_id [String] Source account id
    # @param [Hash] opts the optional parameters
    # @return [SourceAccountResponse]
    def get_source_account(source_account_id, opts = {})
      data, _status_code, _headers = get_source_account_with_http_info(source_account_id, opts)
      data
    end

    # Get details about given source account.
    # Get details about given source account.
    # @param source_account_id [String] Source account id
    # @param [Hash] opts the optional parameters
    # @return [Array<(SourceAccountResponse, Integer, Hash)>] SourceAccountResponse data, response status code and response headers
    def get_source_account_with_http_info(source_account_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_source_account ...'
      end
      # verify the required parameter 'source_account_id' is set
      if @api_client.config.client_side_validation && source_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'source_account_id' when calling FundingManagerApi.get_source_account"
      end
      # resource path
      local_var_path = '/v1/sourceAccounts/{sourceAccountId}'.sub('{' + 'sourceAccountId' + '}', CGI.escape(source_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'SourceAccountResponse' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_source_account\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get details about given source account.
    # Get details about given source account.
    # @param source_account_id [String] Source account id
    # @param [Hash] opts the optional parameters
    # @return [SourceAccountResponseV2]
    def get_source_account_v2(source_account_id, opts = {})
      data, _status_code, _headers = get_source_account_v2_with_http_info(source_account_id, opts)
      data
    end

    # Get details about given source account.
    # Get details about given source account.
    # @param source_account_id [String] Source account id
    # @param [Hash] opts the optional parameters
    # @return [Array<(SourceAccountResponseV2, Integer, Hash)>] SourceAccountResponseV2 data, response status code and response headers
    def get_source_account_v2_with_http_info(source_account_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_source_account_v2 ...'
      end
      # verify the required parameter 'source_account_id' is set
      if @api_client.config.client_side_validation && source_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'source_account_id' when calling FundingManagerApi.get_source_account_v2"
      end
      # resource path
      local_var_path = '/v2/sourceAccounts/{sourceAccountId}'.sub('{' + 'sourceAccountId' + '}', CGI.escape(source_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'SourceAccountResponseV2' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_source_account_v2\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get list of source accounts
    # List source accounts.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :physical_account_name Physical Account Name
    # @option opts [String] :payor_id The account owner Payor ID
    # @option opts [Integer] :page Page number. Default is 1. (default to 1)
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100. (default to 25)
    # @option opts [String] :sort List of sort fields e.g. ?sort&#x3D;name:asc Default is name:asc The supported sort fields are - fundingRef  (default to 'fundingRef:asc')
    # @return [ListSourceAccountResponse]
    def get_source_accounts(opts = {})
      data, _status_code, _headers = get_source_accounts_with_http_info(opts)
      data
    end

    # Get list of source accounts
    # List source accounts.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :physical_account_name Physical Account Name
    # @option opts [String] :payor_id The account owner Payor ID
    # @option opts [Integer] :page Page number. Default is 1.
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100.
    # @option opts [String] :sort List of sort fields e.g. ?sort&#x3D;name:asc Default is name:asc The supported sort fields are - fundingRef 
    # @return [Array<(ListSourceAccountResponse, Integer, Hash)>] ListSourceAccountResponse data, response status code and response headers
    def get_source_accounts_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_source_accounts ...'
      end
      pattern = Regexp.new(/[fundingRef]+[:desc|:asc]/)
      if @api_client.config.client_side_validation && !opts[:'sort'].nil? && opts[:'sort'] !~ pattern
        fail ArgumentError, "invalid value for 'opts[:\"sort\"]' when calling FundingManagerApi.get_source_accounts, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v1/sourceAccounts'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'physicalAccountName'] = opts[:'physical_account_name'] if !opts[:'physical_account_name'].nil?
      query_params[:'payorId'] = opts[:'payor_id'] if !opts[:'payor_id'].nil?
      query_params[:'page'] = opts[:'page'] if !opts[:'page'].nil?
      query_params[:'pageSize'] = opts[:'page_size'] if !opts[:'page_size'].nil?
      query_params[:'sort'] = opts[:'sort'] if !opts[:'sort'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'ListSourceAccountResponse' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_source_accounts\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get list of source accounts
    # List source accounts.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :physical_account_name Physical Account Name
    # @option opts [String] :physical_account_id The physical account ID
    # @option opts [String] :payor_id The account owner Payor ID
    # @option opts [String] :funding_account_id The funding account ID
    # @option opts [Integer] :page Page number. Default is 1. (default to 1)
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100. (default to 25)
    # @option opts [String] :sort List of sort fields e.g. ?sort&#x3D;name:asc Default is name:asc The supported sort fields are - fundingRef, name, balance  (default to 'fundingRef:asc')
    # @return [ListSourceAccountResponseV2]
    def get_source_accounts_v2(opts = {})
      data, _status_code, _headers = get_source_accounts_v2_with_http_info(opts)
      data
    end

    # Get list of source accounts
    # List source accounts.
    # @param [Hash] opts the optional parameters
    # @option opts [String] :physical_account_name Physical Account Name
    # @option opts [String] :physical_account_id The physical account ID
    # @option opts [String] :payor_id The account owner Payor ID
    # @option opts [String] :funding_account_id The funding account ID
    # @option opts [Integer] :page Page number. Default is 1.
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100.
    # @option opts [String] :sort List of sort fields e.g. ?sort&#x3D;name:asc Default is name:asc The supported sort fields are - fundingRef, name, balance 
    # @return [Array<(ListSourceAccountResponseV2, Integer, Hash)>] ListSourceAccountResponseV2 data, response status code and response headers
    def get_source_accounts_v2_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.get_source_accounts_v2 ...'
      end
      pattern = Regexp.new(/[fundingRef|name|balance]+[:desc|:asc]/)
      if @api_client.config.client_side_validation && !opts[:'sort'].nil? && opts[:'sort'] !~ pattern
        fail ArgumentError, "invalid value for 'opts[:\"sort\"]' when calling FundingManagerApi.get_source_accounts_v2, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/v2/sourceAccounts'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'physicalAccountName'] = opts[:'physical_account_name'] if !opts[:'physical_account_name'].nil?
      query_params[:'physicalAccountId'] = opts[:'physical_account_id'] if !opts[:'physical_account_id'].nil?
      query_params[:'payorId'] = opts[:'payor_id'] if !opts[:'payor_id'].nil?
      query_params[:'fundingAccountId'] = opts[:'funding_account_id'] if !opts[:'funding_account_id'].nil?
      query_params[:'page'] = opts[:'page'] if !opts[:'page'].nil?
      query_params[:'pageSize'] = opts[:'page_size'] if !opts[:'page_size'].nil?
      query_params[:'sort'] = opts[:'sort'] if !opts[:'sort'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'ListSourceAccountResponseV2' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#get_source_accounts_v2\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Get Funding Audit Delta
    # Get funding audit deltas for a payor
    # @param payor_id [String] 
    # @param updated_since [DateTime] 
    # @param [Hash] opts the optional parameters
    # @option opts [Integer] :page Page number. Default is 1. (default to 1)
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100. (default to 25)
    # @return [PageResourceFundingPayorStatusAuditResponseFundingPayorStatusAuditResponse]
    def list_funding_audit_deltas(payor_id, updated_since, opts = {})
      data, _status_code, _headers = list_funding_audit_deltas_with_http_info(payor_id, updated_since, opts)
      data
    end

    # Get Funding Audit Delta
    # Get funding audit deltas for a payor
    # @param payor_id [String] 
    # @param updated_since [DateTime] 
    # @param [Hash] opts the optional parameters
    # @option opts [Integer] :page Page number. Default is 1.
    # @option opts [Integer] :page_size Page size. Default is 25. Max allowable is 100.
    # @return [Array<(PageResourceFundingPayorStatusAuditResponseFundingPayorStatusAuditResponse, Integer, Hash)>] PageResourceFundingPayorStatusAuditResponseFundingPayorStatusAuditResponse data, response status code and response headers
    def list_funding_audit_deltas_with_http_info(payor_id, updated_since, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.list_funding_audit_deltas ...'
      end
      # verify the required parameter 'payor_id' is set
      if @api_client.config.client_side_validation && payor_id.nil?
        fail ArgumentError, "Missing the required parameter 'payor_id' when calling FundingManagerApi.list_funding_audit_deltas"
      end
      # verify the required parameter 'updated_since' is set
      if @api_client.config.client_side_validation && updated_since.nil?
        fail ArgumentError, "Missing the required parameter 'updated_since' when calling FundingManagerApi.list_funding_audit_deltas"
      end
      # resource path
      local_var_path = '/v1/deltas/fundings'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'payorId'] = payor_id
      query_params[:'updatedSince'] = updated_since
      query_params[:'page'] = opts[:'page'] if !opts[:'page'].nil?
      query_params[:'pageSize'] = opts[:'page_size'] if !opts[:'page_size'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] 

      # return_type
      return_type = opts[:return_type] || 'PageResourceFundingPayorStatusAuditResponseFundingPayorStatusAuditResponse' 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#list_funding_audit_deltas\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Set notifications
    # Set notifications for a given source account
    # @param source_account_id [String] Source account id
    # @param set_notifications_request [SetNotificationsRequest] Body to included minimum balance to set
    # @param [Hash] opts the optional parameters
    # @return [nil]
    def set_notifications_request(source_account_id, set_notifications_request, opts = {})
      set_notifications_request_with_http_info(source_account_id, set_notifications_request, opts)
      nil
    end

    # Set notifications
    # Set notifications for a given source account
    # @param source_account_id [String] Source account id
    # @param set_notifications_request [SetNotificationsRequest] Body to included minimum balance to set
    # @param [Hash] opts the optional parameters
    # @return [Array<(nil, Integer, Hash)>] nil, response status code and response headers
    def set_notifications_request_with_http_info(source_account_id, set_notifications_request, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.set_notifications_request ...'
      end
      # verify the required parameter 'source_account_id' is set
      if @api_client.config.client_side_validation && source_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'source_account_id' when calling FundingManagerApi.set_notifications_request"
      end
      # verify the required parameter 'set_notifications_request' is set
      if @api_client.config.client_side_validation && set_notifications_request.nil?
        fail ArgumentError, "Missing the required parameter 'set_notifications_request' when calling FundingManagerApi.set_notifications_request"
      end
      # resource path
      local_var_path = '/v1/sourceAccounts/{sourceAccountId}/notifications'.sub('{' + 'sourceAccountId' + '}', CGI.escape(source_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(set_notifications_request) 

      # return_type
      return_type = opts[:return_type] 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#set_notifications_request\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # Transfer Funds between source accounts
    # Transfer funds between source accounts for a Payor. The 'from' source account is identified in the URL, and is the account which will be debited. The 'to' (destination) source account is in the body, and is the account which will be credited. Both source accounts must belong to the same Payor. There must be sufficient balance in the 'from' source account, otherwise the transfer attempt will fail.
    # @param source_account_id [String] The &#39;from&#39; source account id, which will be debited
    # @param transfer_request [TransferRequest] Body
    # @param [Hash] opts the optional parameters
    # @return [nil]
    def transfer_funds(source_account_id, transfer_request, opts = {})
      transfer_funds_with_http_info(source_account_id, transfer_request, opts)
      nil
    end

    # Transfer Funds between source accounts
    # Transfer funds between source accounts for a Payor. The &#39;from&#39; source account is identified in the URL, and is the account which will be debited. The &#39;to&#39; (destination) source account is in the body, and is the account which will be credited. Both source accounts must belong to the same Payor. There must be sufficient balance in the &#39;from&#39; source account, otherwise the transfer attempt will fail.
    # @param source_account_id [String] The &#39;from&#39; source account id, which will be debited
    # @param transfer_request [TransferRequest] Body
    # @param [Hash] opts the optional parameters
    # @return [Array<(nil, Integer, Hash)>] nil, response status code and response headers
    def transfer_funds_with_http_info(source_account_id, transfer_request, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: FundingManagerApi.transfer_funds ...'
      end
      # verify the required parameter 'source_account_id' is set
      if @api_client.config.client_side_validation && source_account_id.nil?
        fail ArgumentError, "Missing the required parameter 'source_account_id' when calling FundingManagerApi.transfer_funds"
      end
      # verify the required parameter 'transfer_request' is set
      if @api_client.config.client_side_validation && transfer_request.nil?
        fail ArgumentError, "Missing the required parameter 'transfer_request' when calling FundingManagerApi.transfer_funds"
      end
      # resource path
      local_var_path = '/v2/sourceAccounts/{sourceAccountId}/transfers'.sub('{' + 'sourceAccountId' + '}', CGI.escape(source_account_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      header_params['Content-Type'] = @api_client.select_header_content_type(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:body] || @api_client.object_to_http_body(transfer_request) 

      # return_type
      return_type = opts[:return_type] 

      # auth_names
      auth_names = opts[:auth_names] || ['OAuth2']

      new_options = opts.merge(
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: FundingManagerApi#transfer_funds\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
